/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "spring.h"

Spring::Spring( QWidget *parent )
		: QGLWidget( parent )
{
	// setMouseTracking( TRUE );

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(oneStepDeformation()) );

	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

	s.f = 0.0;
	s.k = 11.11111;
	s.m = 1.0;
	s.b = 2.66667;

	s.dt = 0.033;

	s.k1 = 2.0 - s.dt*(s.k*s.dt + s.b)/s.m;
	s.k2 = 1.0 - s.dt*s.b/s.m;
	s.k3 = (s.f * s.dt * s.dt)/s.m;
	
	s.x_1 = s.x = s.x1 = 0.0;
}



//
// This method puts the initial coordinates each time window is resized
//
void Spring::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if( w <= h )
		glOrtho( -0.125, 0.125, -0.05/aspect, 0.05/aspect, -1.0, 1.0);
	else
		glOrtho(-0.125*aspect, 0.125*aspect, -0.05, 0.05, -1.0, 1.0);
}

void Spring::paintGL( void )
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */


	// glRotatef ( angle , 1.0, 1.0, 1.0);

	glLineWidth( 4.0 );
	showSpring ( );	

	swapBuffers( );
}

void Spring::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);
}

void Spring::showSpring(  )
{
	double v;

	glBegin( GL_LINES );

	glColor3f( 0.0, 0.0, 1.0);

	glVertex2d(-0.08, 0.025 ); 
	glVertex2d(-0.08,-0.025 ); 

	glColor3f( 1.0, 0.0, 0.0);
	glVertex2d(-0.08, 0.0 ); 

	v = -0.02+s.x1;
	glVertex2d( v, 0.0 ); 

	glEnd(  );

	glBegin( GL_QUADS );
		glColor3f( 1.0, 0.0, 0.0);
		glVertex2d( v-0.005,-0.005 );
		glVertex2d( v+0.005,-0.005 );
		glVertex2d( v+0.005, 0.005 );
		glVertex2d( v-0.005,+0.005 );
	glEnd(  );
}

void Spring::oneStepDeformation( void )
{
	s.x_1 = s.x;
	s.x = s.x1;
	s.x1 = s.k1*s.x - s.k2*s.x_1 + s.k3;

	updateGL();
}
